@@ -36,5 +36,8 @@ strndup(const char *s, size_t n)
 	new_s[len] = '\0';
 	return new_s;
 }
-
+#else /* #ifndef HAVE_STRNDUP */
+int dummy_dummy_required_for_solaris_do_not_use(void)
+{
+}
 #endif /* #ifndef HAVE_STRNDUP */
