@@ -70,10 +70,10 @@ int advstats_lit_parser_calls[ADVSTATS_MAX_ENTITIES];
 static struct ln_parser_info parser_lookup_table[] = {
 	PARSER_ENTRY("literal", Literal, 4),
 	PARSER_ENTRY("repeat", Repeat, 4),
-	PARSER_ENTRY_NO_DATA("date-rfc3164", RFC3164Date, 8),
-	PARSER_ENTRY_NO_DATA("date-rfc5424", RFC5424Date, 8),
-	PARSER_ENTRY_NO_DATA("number", Number, 16),
-	PARSER_ENTRY_NO_DATA("float", Float, 16),
+	PARSER_ENTRY("date-rfc3164", RFC3164Date, 8),
+	PARSER_ENTRY("date-rfc5424", RFC5424Date, 8),
+	PARSER_ENTRY("number", Number, 16),
+	PARSER_ENTRY("float", Float, 16),
 	PARSER_ENTRY("hexnumber", HexNumber, 16),
 	PARSER_ENTRY_NO_DATA("kernel-timestamp", KernelTimestamp, 16),
 	PARSER_ENTRY_NO_DATA("whitespace", Whitespace, 4),
@@ -415,7 +415,7 @@ for(int i = 0 ; i < dag->nparsers ; ++i) { /* TODO: remove when confident enough
 
 
 static void
-deleteComponentID(struct ln_pdag *const __restrict dag)
+deleteComponentID(struct ln_pdag *const __restrict__ dag)
 {
 	free((void*)dag->rb_id);
 	dag->rb_id = NULL;
